#include <fstream>
#include <mimas/mm_image.h>
#include <mimas/mm_image_fileinput.h>
#include <mimas/mm_image_fileoutput.h>
#include <mimas/mm_rgba.h>
#include <mimas/mm_image_op.h>

using namespace mimas;
using namespace std;

template< class T >
struct _add: public binary_function< T, T, T >
{
  T operator()( const T &a, const T &b ) const
  {
    return T( a.getRed() + b.getRed(),
              a.getGreen() + b.getGreen(),
              a.getBlue() + b.getBlue() );
  }
};

template< typename T >
struct _div: public unary_function< T, T >
{
  _div( int _divisor ): divisor(_divisor), offset(_divisor/2) {}
  T operator()( const T &x ) const
  {
    return T( ( x.getRed() + offset ) / divisor,
              ( x.getGreen() + offset ) / divisor,
              ( x.getBlue() + offset ) / divisor );
  };
  int divisor;
  int offset;
};

MM_ARRAY_ARRAY_OP( colourAddIt, colourAdd, _add )
MM_IMAGE_IMAGE_OP( colourAddIt, colourAdd )


mm_image< mm_rgba< int > > colourDiv( const mm_image< mm_rgba< int > > &image,
                           int divisor )
{
  mm_image< mm_rgba< int > > retVal;
  retVal.init( image.getWidth(), image.getHeight() );
  _div< mm_rgba< int > > div( divisor );
  mm_array_apply_extern< _div< mm_rgba< int > >, 2 > apply( div );
  apply( retVal.rawData().origin(), image.rawData().origin(),
         retVal.rawData().shape(), image.rawData().shape(),
         retVal.rawData().strides(), image.rawData().strides() );
  return retVal;
}

int main( int argc, char *argv[] )
{
  int retval = 0;
  try {
    MMERROR( argc >= 3, mm_exception, ,
             "Syntax: " << argv[0] << " infile1 ... outfile" );
    mm_image< mm_rgba< int > > out;
    for ( int i=1; i<argc-1; i++ ) {
      ifstream file( argv[i] );
      mm_image< mm_rgba< int > > in;
      MMERROR( file >> in, mm_exception, ,
               "Error reading file \"" << argv[i] << "\"." );
      if ( !out.initialised() ) {
        out.init(  in.getWidth(), in.getHeight() );
      } else
        MMERROR( in.getWidth() == out.getWidth() &&
                 in.getHeight() == out.getHeight(), mm_exception, ,
                 "Image in file \"" << argv[i] << "\" was of size "
                 << in.getWidth() << 'x' << in.getHeight()
                 << ", but should be of size " << out.getWidth() << 'x'
                 << out.getHeight() );
      colourAddIt( out, in );
    };
    ofstream file( argv[argc-1] );
    file << colourDiv( out, argc - 2 );
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retval = 1;
  };
  return retval;
}
