#ifndef __CAMERAVIEW_H
#define __CAMERAVIEW_H

#include <qgl.h>
#include <mimas/mm_image.h>
#include <mimas/mm_image_firewireinput.h>
#include <mimas/mm_rgba.h>

// Field of view in micrometers (when using the 5x objective)
// is 1081.829 um.
#define FOV5X ( 38546.744 - 37467.126 )
#define FOV5Y ( 14705.8  - 13902.644 )
#define RESOLUTIONX 640
#define RESOLUTIONY 480

class QScrollView;

class CameraView: public QGLWidget
{
  Q_OBJECT
 public:
  ///
  typedef enum { Colour = 0, GrayScale, Sobel } Mode;
  /// Constructor.
  CameraView( QScrollView *_scrollView, const char *name = NULL );
  ///
  virtual ~CameraView(void);
  ///
  mimas::mm_image< mimas::mm_rgba< unsigned char > > image;
  ///
  mimas::mm_image_firewireinput< mimas::mm_rgba< unsigned char > > input;
  ///
  void refresh(void);
 public slots:
  /** Set display mode.
      @see mode. */
  void setMode(int);
  /// Set camera shutter.
  void setShutter(int);
  /// Set camera gain.
  void setGain(int);
  /// Set white balance.
  void setWhiteBalance(int);
  /// Set zoom of display.
  void setZoom(int);
  ///
  void useThreshold(bool);
  ///
  void setThreshold(int);
  ///
  void setGrabbing(bool);
  ///
  void setGrabFilePrefix(const QString&);
 signals:
  ///
  void startDrag( int );
  /// Request movement of microscope's table.
  void mouseDragged( double, double );
  ///
  void stopDrag();
  /// Request rotation of nosepiece.
  void wheelTick( int );
 protected:
  ///
  virtual void mousePressEvent( QMouseEvent *e );
  ///
  virtual void mouseMoveEvent( QMouseEvent *e );
  ///
  virtual void mouseReleaseEvent ( QMouseEvent *e );
  ///
  virtual void timerEvent( QTimerEvent *e );
  ///
  virtual void wheelEvent( QWheelEvent *e );
  ///
  void initializeGL(void);
  ///
  void resizeGL( int w, int h );
  ///
  void paintGL(void);
  /// Adapt size of widget to current image.
  void adaptSize(void);
  ///
  QScrollView *scrollView;
  /// Zoom-factor for display.
  double zoom;
  ///
  Mode mode;
  ///
  bool mousePressed;
  ///
  int mouseX;
  ///
  int mouseY;
  ///
  int wheelFrac;
  ///
  bool doThreshold;
  ///
  int thresholdValue;
  ///
  bool grabbing;
  ///
  int grabCounter;
  ///
  std::string grabFilePrefix;
};

#endif
