#include <iostream>
#include <cassert>
#include "drive.h"
#include "microscope.h"

using namespace mimas;
using namespace std;

void Drive::init( Microscope *_microscope,
                  const string &conversionRequest,
                  const string &_readCommand,
                  const string &_moveRelCommand,
                  const string &_moveCommand,
                  const string &_manualCommand,
                  const string &_continuousMoveCommand,
                  const string &_stopCommand ) throw (mm_exception)
{
  microscope = _microscope;
  conversion = microscope->readDouble( conversionRequest );
  assert( conversion != 0.0 );
  readCommand = _readCommand;
  moveRelCommand = _moveRelCommand;
  moveCommand = _moveCommand;
  manualCommand = _manualCommand;
  continuousMoveCommand = _continuousMoveCommand;
  stopCommand = _stopCommand;
}

double Drive::getPosition(void) const throw (mm_exception)
{
  assert( microscope != NULL );
  assert( conversion != 0.0 );
  return microscope->readDouble( readCommand ) * conversion;
}

void Drive::setPositionRel( double shift, bool wait ) throw (mm_exception)
{
  assert( microscope != NULL );
  assert( conversion != 0.0 );
  microscope->sendInt( moveRelCommand, (int)( shift / conversion ), wait );
}

void Drive::setPosition( double position, bool wait ) throw (mm_exception)
{
  assert( microscope != NULL );
  assert( conversion != 0.0 );
  microscope->sendInt( moveCommand, (int)( position / conversion ), wait );
}

void Drive::setManual( bool on ) throw (mm_exception)
{
  assert( microscope != NULL );
  microscope->sendInt( manualCommand, on ? 1 : 0, false );
  // Ignore return-value.
  microscope->readString();
}

void Drive::move( double speed ) throw (mm_exception)
{
  assert( microscope != NULL );
  microscope->sendInt( continuousMoveCommand,
                       (int)( speed / conversion ), true );
}

void Drive::stop(void) throw (mm_exception)
{
  assert( microscope != NULL );
  microscope->command( stopCommand );
}
