#ifndef __DRIVE_H
#define __DRIVE_H

#include <mimas/mm_exception.h>

// Desired speed in pixel/second (doesn't seem to be calibrated properly)
#define SPEEDXY 1200
#define SPEEDZ 1200

class Microscope;

class Drive
{
 public:
  Drive(void): microscope(NULL), conversion(0.0) {}
  Drive( Microscope *_microscope,
         const std::string &conversionRequest,
         const std::string &_readCommand,
         const std::string &_moveRelCommand,
         const std::string &_moveCommand,
         const std::string &_manualCommand,
         const std::string &_continuousMoveCommand,
         const std::string &_stopCommand )
    throw (mimas::mm_exception)
  { init( _microscope, conversionRequest, _readCommand, _moveRelCommand,
          _moveCommand, _manualCommand, _continuousMoveCommand,
          _stopCommand ); }
  void init( Microscope *_microscope, const std::string &conversionRequest,
             const std::string &_readCommand,
             const std::string &_moveRelCommand,
             const std::string &_moveCommand,
             const std::string &_manualCommand,
             const std::string &_continuousMoveCommand,
             const std::string &_stopCommand )
    throw (mimas::mm_exception);
  double getPosition(void) const throw (mimas::mm_exception);
  void setPositionRel( double shift, bool wait = true )
    throw (mimas::mm_exception);
  void setPosition( double position, bool wait = true )
    throw (mimas::mm_exception);
  void setManual( bool on = true ) throw (mimas::mm_exception);
  void move( double speed ) throw (mimas::mm_exception);
  void stop(void) throw (mimas::mm_exception);
 protected:
  Microscope *microscope;
  double conversion;
  std::string readCommand;
  std::string moveRelCommand;
  std::string moveCommand;
  std::string manualCommand;
  std::string continuousMoveCommand;
  std::string stopCommand;
};

#endif
