/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

const int LeicaWindow::pollInterval = 2000;

void LeicaWindow::init()
{
  // Set validator for stepping of focus-stack.
  stepSizeValidator = new QDoubleValidator( stepSizeEdit );
  stepSizeValidator->setRange( 0.1, 500.0, 3 );
  stepSizeEdit->setValidator( stepSizeValidator );

  // Access microscope.
  microscope = MicroscopePtr( new Microscope );

  // Insert camera-view in scrollview and connect it.
  cameraView = new CameraView( scrollView, "cameraView" );
  scrollView->addChild( cameraView );

  connect( cameraView, SIGNAL(startDrag(int)),
           this, SLOT(startTable(int)) );
  connect( cameraView, SIGNAL(mouseDragged(double,double)),
           this, SLOT(moveTable(double,double)) );
  connect( cameraView, SIGNAL(stopDrag()),
           this, SLOT(stopTable()) );
  connect( cameraView, SIGNAL(wheelTick(int)),
           this, SLOT(rotateNosePiece(int)) );
  connect( thresholdCheckBox, SIGNAL(toggled(bool)),
           cameraView, SLOT(useThreshold(bool)) );
  connect( thresholdSpinBox, SIGNAL(valueChanged(int)),
           cameraView, SLOT(setThreshold(int)) );
  connect( grabButton, SIGNAL(toggled(bool)),
           cameraView, SLOT(setGrabbing(bool)) );
  connect( grabFileEdit, SIGNAL(textChanged(const QString&)),
           cameraView, SLOT(setGrabFilePrefix(const QString&)) );
 

  // Set limits of shutter-dial and connect it.
  dc1394_feature_info shutter =
    cameraView->input.get_feature( FEATURE_SHUTTER );
  shutterDial->setMinValue( shutter.min );
  shutterDial->setMaxValue( shutter.max );
  shutterDial->setValue( shutter.value );
  shutterDial->setPageStep( ( shutter.max - shutter.min ) / 10 );
  shutterDial->setLineStep( ( shutter.max - shutter.min ) / 50 );
  connect( shutterDial, SIGNAL(valueChanged(int)),
           cameraView, SLOT(setShutter(int)));

  // Set limits of gain-dial and connect it.
  dc1394_feature_info gain =
    cameraView->input.get_feature( FEATURE_GAIN );
  gainDial->setMinValue( gain.min );
  gainDial->setMaxValue( gain.max );
  gainDial->setValue( gain.value );
  gainDial->setPageStep( ( gain.max - gain.min ) / 10 );
  gainDial->setLineStep( ( gain.max - gain.min ) / 50 );
  connect( gainDial, SIGNAL(valueChanged(int)),
           cameraView, SLOT(setGain(int)) );

  // Set limits of white-balance-dial and connect it.
  dc1394_feature_info balance =
    cameraView->input.get_feature( FEATURE_WHITE_BALANCE );
  balanceDial->setMinValue( balance.min );
  balanceDial->setMaxValue( balance.max );
  balanceDial->setValue( balance.min );// balance.value didn't work.
  balanceDial->setPageStep( ( balance.max - balance.min ) / 10 );
  balanceDial->setLineStep( ( balance.max - balance.min ) / 50 );
  connect( balanceDial, SIGNAL(valueChanged(int)),
           cameraView, SLOT(setWhiteBalance(int)) );


  // Set limits of lamp-dial and connect it.
  int
    minLamp = microscope->getLamp().getMinValue(),
    maxLamp = microscope->getLamp().getMaxValue();
  lampDial->setMinValue( minLamp );
  lampDial->setMaxValue( maxLamp );
  lampDial->setPageStep( ( maxLamp - minLamp ) / 10 );
  lampDial->setLineStep( ( maxLamp - minLamp ) / 50 );
  connect( lampDial, SIGNAL(valueChanged(int)),
           this, SLOT(setLamp(int)) );

  // Connect zoom-slider.
  connect( zoomSlider, SIGNAL(valueChanged(int)),
           cameraView, SLOT(setZoom(int)) );

  // Polling of microscope-status.
  // pollTimer = -1;
  pollTimer = startTimer( pollInterval );

  // Initialise variable for mouse-dragging.
  tableMove = 0;
}


void LeicaWindow::setGrabFile( const QString &_grabFileName )
{
  grabButton->setEnabled( !_grabFileName.isEmpty() );
}

void LeicaWindow::updateLamp()
{
  try {

    lampDial->setValue( microscope->getLamp().getValue() );

  } catch ( mimas::mm_exception &e ) {

    lampDial->setValue( lampDial->minValue() );
    throw e;
  };
}

void LeicaWindow::updatePosition()
{
  try {

    std::ostringstream x, y, z;
    x << std::setprecision( 3 ) << std::fixed << std::setw( 10 )
      << microscope->getDriveX().getPosition() << " um";
    y << std::setprecision( 3 ) << std::fixed << std::setw( 10 )
      << microscope->getDriveY().getPosition() << " um";
    z << std::setprecision( 3 ) << std::fixed << std::setw( 10 )
      << microscope->getDriveZ().getPosition() << " um";

    xPosLabel->setText( x.str() );
    yPosLabel->setText( y.str() );
    zPosLabel->setText( z.str() );

  } catch ( mimas::mm_exception &e ) {

    xPosLabel->setText( "---" );
    yPosLabel->setText( "---" );
    zPosLabel->setText( "---" );
    throw e;

  };
}

void LeicaWindow::timerEvent( QTimerEvent *e )
{
  if ( e->timerId() == pollTimer ) {

    try {

      updateLamp();
      updatePosition();

    } catch ( std::exception &e ) {
      handleError( "reading status", e.what() );
    };
  };
}


void LeicaWindow::setLamp( int value )
{
  try {

    microscope->getLamp().setValue( value );
    cameraView->refresh();

  } catch ( std::exception &e ) {
    handleError( "accessing lamp", e.what() );
  };
}

void LeicaWindow::startTable( int mode )
{
  try {
    if ( mode == 1 ) {
      microscope->getDriveX().setManual( false );
      microscope->getDriveY().setManual( false );
      referenceX = microscope->getDriveX().getPosition();
      referenceY = microscope->getDriveY().getPosition();
      tableMove = 1;
    } else {// mode == 2
      microscope->getDriveZ().setManual( false );
      referenceZ = microscope->getDriveZ().getPosition();
      tableMove = 2;
    };
  } catch ( std::exception &e ) {
    handleError( "accessing table", e.what() );
  };
}

void LeicaWindow::moveTable( double deltaX, double deltaY )
{
  try {
  
      double magnification =
        microscope->getNosePiece().getMagnification
        ( microscope->getNosePiece().getPosition() );
      
      double
        scaleX = FOV5X / RESOLUTIONX * 5 / magnification,
        scaleY = FOV5Y / RESOLUTIONY * 5 / magnification;

      if ( tableMove == 1 ) {

        microscope->getDriveX().
          setPosition( referenceX + deltaX * scaleX, false );
        microscope->getDriveY().
          setPosition( referenceY + deltaY * scaleY, false );

        microscope->readString();
        microscope->readString();
        
        updatePosition();

      } else {// tableMove == 2
        
        microscope->getDriveZ().setPosition( referenceZ + deltaY * scaleY );
        
        /* for ( int pending = 6; pending > 0; pending-- ) {
          do {
            cameraView->refresh();
          } while ( !microscope->inputAvailable() );          
          microscope->readChar();
          }; */

        updatePosition();

      };

  } catch ( std::exception &e ) {
    handleError( "moving table", e.what() );
  };

}

void LeicaWindow::stopTable()
{
  try {
    if ( tableMove == 1 ) {
      microscope->getDriveX().setManual( true );
      microscope->getDriveY().setManual( true );
    } else {// tableMove == 2
      microscope->getDriveZ().setManual( true );
    };
    tableMove = 0;

  } catch ( std::exception &e ) {
    handleError( "setting table to manual control", e.what() );
  };
}

void LeicaWindow::rotateNosePiece( int change )
{
  try {

    microscope->getNosePiece().setPositionRel( change );

  } catch ( std::exception &e ) {
    handleError( "rotating nosepiece", e.what() );
  };
}


void LeicaWindow::chooseGrabFile()
{
  // Stop grabbing.
  if ( pollTimer != -1 ) killTimer( pollTimer );
  cameraView->killTimers();

  // Open file-dialog.
  QString fileName =
    QFileDialog::getSaveFileName( grabFileEdit->text(),
                                  QString( "PPM (*.ppm);;All Files (*)" ),
                                  this, "fileDialog", "Save Frame" );
  if (  !fileName.isEmpty() ) {

    if ( fileName.right( 8 ) == "0000.ppm" ) {
      QMessageBox::warning( this, "Warning",
                            "Removing 0000.ppm-extension.",
                            QMessageBox::Ok, 0 );
      fileName = fileName.left( fileName.length() - 8 );
    } else if ( fileName.right( 4 ) == ".ppm" ) {
      QMessageBox::warning( this, "Warning",
                            "Removing .ppm-extension.",
                            QMessageBox::Ok, 0 );
      fileName = fileName.left( fileName.length() - 4 );
    };

    grabFileEdit->setText( fileName );

  };

  // Continue grabbing.
  if ( pollTimer != -1 ) pollTimer = startTimer( pollInterval );
  cameraView->startTimer( 0 );
}


void LeicaWindow::setMode( int _mode )
{
  cameraView->setMode( _mode );
  thresholdCheckBox->setEnabled( _mode == 2 );
  thresholdSpinBox->setEnabled( _mode == 2 );
}


void LeicaWindow::setPolling( bool _on )
{
  if ( _on ) {
    // Start timer.
    pollTimer = startTimer( pollInterval );
  } else {
    killTimer( pollTimer );
    pollTimer = -1;
  };
}


void LeicaWindow::enableStackGrab()
{
  QString text = stepSizeEdit->text();
  int dummy = 0;

  stackButton->setEnabled
    ( stepSizeValidator->validate( text, dummy ) == QValidator::Acceptable &&
      !stackFileEdit->text().isEmpty() );
}


void LeicaWindow::chooseStackGrabFile()
{
  // Stop grabbing.
  if ( pollTimer != -1 ) killTimer( pollTimer );
  cameraView->killTimers();

  QString fileName =
    QFileDialog::getSaveFileName( stackFileEdit->text(),
                                  QString( "PPM (*.ppm);;All Files (*)" ),
                                  this, "fileDialog", "Save Frame" );
  if ( !fileName.isEmpty() ) {

    if ( fileName.right( 8 ) == "0000.ppm" ) {
      QMessageBox::warning( this, "Warning",
                            "Removing 0000.ppm-extension.",
                            QMessageBox::Ok, 0 );
      fileName = fileName.left( fileName.length() - 8 );
    } else if ( fileName.right( 4 ) == ".ppm" ) {
      QMessageBox::warning( this, "Warning",
                            "Removing .ppm-extension.",
                            QMessageBox::Ok, 0 );
      fileName = fileName.left( fileName.length() - 4 );
    };

    stackFileEdit->setText( fileName );
    enableStackGrab();

  };   

  // Continue grabbing.
  if ( pollTimer != -1 ) pollTimer = startTimer( pollInterval );
  cameraView->startTimer( 0 );
}


void LeicaWindow::grabStack()
{
  try { 

    DisplayWaitCursor w;

    microscope->getDriveX().setManual( false );
    microscope->getDriveY().setManual( false );
    microscope->getDriveZ().setManual( false );

    double
      z0 = microscope->getDriveZ().getPosition(),
      zStep = atof( stepSizeEdit->text() );

    for ( int i=0; i<numImagesSpinBox->value(); i++ ) {
      
      // usleep( 1000000 );
      cameraView->refresh();

      std::ostringstream fileName;
      fileName << stackFileEdit->text() << std::setfill('0') << std::setw( 4 )
               << i << ".ppm";
      
      std::ofstream f( fileName.str().c_str(), std::ios::binary );
      MMERROR( f << cameraView->image, mimas::mm_exception, ,
               "Couldn't write file \"" << fileName.str() << "\"." );
      
      microscope->getDriveZ().setPosition( z0 - zStep * ( i + 1 ), false );
      // Sometimes funny responses.
      microscope->readString();

    };

    microscope->getDriveZ().setPosition( z0 );

    microscope->getDriveX().setManual( true );
    microscope->getDriveY().setManual( true );
    microscope->getDriveZ().setManual( true );

  } catch ( std::exception &e ) {

    microscope->getDriveX().setManual( true );
    microscope->getDriveY().setManual( true );
    microscope->getDriveZ().setManual( true );

    handleError( "grabbing focus set", e.what() );
  };
}


void LeicaWindow::upStart()
{
  try {
    double magnification =
      microscope->getNosePiece().getMagnification
      ( microscope->getNosePiece().getPosition() );
    
    double scaleX = FOV5X / RESOLUTIONX * 5 / magnification;
    
    microscope->getDriveZ().move( +scaleX * SPEEDZ );

  } catch ( std::exception &e ) {
    handleError( "starting z-drive", e.what() );
  };
}


void LeicaWindow::upStop()
{
  try { 
    microscope->getDriveZ().stop();
  } catch ( std::exception &e ) {
    handleError( "stopping z-drive", e.what() );
  };
}


void LeicaWindow::downStart()
{
  try {
    double magnification =
      microscope->getNosePiece().getMagnification
      ( microscope->getNosePiece().getPosition() );
    
    double scaleX = FOV5X / RESOLUTIONX * 5 / magnification;
    
    microscope->getDriveZ().move( -scaleX * SPEEDZ );

  } catch ( std::exception &e ) {
    handleError( "starting z-drive", e.what() );
  };
}


void LeicaWindow::downStop()
{
  try { 
    microscope->getDriveZ().stop();
  } catch ( std::exception &e ) {
    handleError( "stopping z-drive", e.what() );
  };
}


void LeicaWindow::leftStart()
{
  try {

    double magnification =
      microscope->getNosePiece().getMagnification
      ( microscope->getNosePiece().getPosition() );
    
    double scaleX = FOV5X / RESOLUTIONX * 5 / magnification;
    
    microscope->getDriveX().move( +scaleX * SPEEDXY );

  } catch ( std::exception &e ) {
    handleError( "starting x-drive", e.what() );
  };
}


void LeicaWindow::leftStop()
{
  try { 
    microscope->getDriveX().stop();
  } catch ( std::exception &e ) {
    handleError( "stopping x-drive", e.what() );
  };
}


void LeicaWindow::rightStart()
{
  try {

    double magnification =
      microscope->getNosePiece().getMagnification
      ( microscope->getNosePiece().getPosition() );
    
    double scaleX = FOV5X / RESOLUTIONX * 5 / magnification;
    
    microscope->getDriveX().move( -scaleX * SPEEDXY );

  } catch ( std::exception &e ) {
    handleError( "starting x-drive", e.what() );
  };
}


void LeicaWindow::rightStop()
{
  try { 
    microscope->getDriveX().stop();
  } catch ( std::exception &e ) {
    handleError( "stopping x-drive", e.what() );
  };
}


void LeicaWindow::handleError( const std::string &context,
                               const std::string &msg )
{
  if ( pollTimer != -1 ) killTimer( pollTimer );
  if ( QMessageBox::critical( this, "Error",
                              QString( "Error %1: %2\n"
                                       "Exit application?" ).
                              arg( context ).arg( msg ),
                              QMessageBox::Yes,
                              QMessageBox::No | QMessageBox::Default ) ==
       QMessageBox::Yes )
    qApp->quit();
  else if ( pollTimer != -1 )
    pollTimer = startTimer( pollInterval );
}


void LeicaWindow::forwardStart()
{
  try {

    double magnification =
      microscope->getNosePiece().getMagnification
      ( microscope->getNosePiece().getPosition() );
    
    double scaleY = FOV5Y / RESOLUTIONY * 5 / magnification;
    
    microscope->getDriveY().move( +scaleY * SPEEDXY );

  } catch ( std::exception &e ) {
    handleError( "starting y-drive", e.what() );
  };
}


void LeicaWindow::forwardStop()
{
  try { 
    microscope->getDriveY().stop();
  } catch ( std::exception &e ) {
    handleError( "stopping y-drive", e.what() );
  };
}


void LeicaWindow::backwardStart()
{
  try {

    double magnification =
      microscope->getNosePiece().getMagnification
      ( microscope->getNosePiece().getPosition() );
    
    double scaleY = FOV5Y / RESOLUTIONY * 5 / magnification;
    
    microscope->getDriveY().move( -scaleY * SPEEDXY );

  } catch ( std::exception &e ) {
    handleError( "starting y-drive", e.what() );
  };
}


void LeicaWindow::backwardStop()
{
  try { 
    microscope->getDriveY().stop();
  } catch ( std::exception &e ) {
    handleError( "stopping y-drive", e.what() );
  };
}


void LeicaWindow::rightPage()
{
  try {
  
      double magnification =
        microscope->getNosePiece().getMagnification
        ( microscope->getNosePiece().getPosition() );
      
      double scaleX = FOV5X / RESOLUTIONX * 5 / magnification;

      microscope->getDriveX().
        setPositionRel( -RESOLUTIONX * scaleX );
      
      updatePosition();

  } catch ( std::exception &e ) {
    handleError( "moving table to the right", e.what() );
  };
}


void LeicaWindow::leftPage()
{
  try {
  
      double magnification =
        microscope->getNosePiece().getMagnification
        ( microscope->getNosePiece().getPosition() );
      
      double scaleX = FOV5X / RESOLUTIONX * 5 / magnification;

      microscope->getDriveX().
        setPositionRel( +RESOLUTIONX * scaleX );
      
      updatePosition();

  } catch ( std::exception &e ) {
    handleError( "moving table to the left", e.what() );
  };
}


void LeicaWindow::upPage()
{
  try {
  
      double magnification =
        microscope->getNosePiece().getMagnification
        ( microscope->getNosePiece().getPosition() );
      
      double scaleY = FOV5Y / RESOLUTIONY * 5 / magnification;

      microscope->getDriveY().
        setPositionRel( +RESOLUTIONY * scaleY );
      
      updatePosition();

  } catch ( std::exception &e ) {
    handleError( "moving table up", e.what() );
  };
}


void LeicaWindow::downPage()
{
  try {
  
      double magnification =
        microscope->getNosePiece().getMagnification
        ( microscope->getNosePiece().getPosition() );
      
      double scaleY = FOV5Y / RESOLUTIONY * 5 / magnification;

      microscope->getDriveY().
        setPositionRel( -RESOLUTIONY * scaleY );
      
      updatePosition();

  } catch ( std::exception &e ) {
    handleError( "moving table down", e.what() );
  };
}
