#ifndef __MICROSCOPE_H
#define __MICROSCOPE_H

#include <boost/smart_ptr.hpp>
#include <mimas/mm_exception.h>
#include <SerialStream.h>
#include "drive.h"
#include "lamp.h"
#include "nosePiece.h"

#define SERIALPORT "/dev/ttyS0"
// #define SERIALPORT "/dev/ttyUSB0"


class Microscope
{
 public:
  Microscope( const std::string &_device = SERIALPORT )
    throw (mimas::mm_exception);
  void probe(void) throw (mimas::mm_exception);
  Drive &getDriveX(void) { return driveX; }
  Drive &getDriveY(void) { return driveY; }
  Drive &getDriveZ(void) { return driveZ; }
  Lamp &getLamp(void) { return lamp; }
  NosePiece &getNosePiece(void) { return nosePiece; }
  std::string readLine(void) throw (mimas::mm_exception);
  std::string readLine( const std::string &command )
    throw (mimas::mm_exception);
  std::string readString(void) throw (mimas::mm_exception);
  std::string readString( const std::string &command )
    throw (mimas::mm_exception);
  double readDouble( const std::string &command ) throw (mimas::mm_exception);
  int readInt( const std::string &command ) throw (mimas::mm_exception);
  int readChar(void) throw (mimas::mm_exception);
  void sendInt( const std::string &command, int value, bool wait = true )
    throw (mimas::mm_exception);
  bool inputAvailable(void) { return port.rdbuf()->in_avail() > 0; }
  void command( const std::string &cmd ) throw (mimas::mm_exception);
  static const std::string IDENTIFICATION;
 protected:
  void query( const std::string &command ) throw (mimas::mm_exception);
  void checkEcho( const std::string &command ) throw (mimas::mm_exception);
  LibSerial::SerialStream port;
  std::string device;
  Drive driveX, driveY, driveZ;
  Lamp lamp;
  NosePiece nosePiece;
};

typedef boost::shared_ptr< Microscope > MicroscopePtr;

#endif
