#include <cassert>
#include <sstream>
#include "microscope.h"
#include "nosePiece.h"

using namespace mimas;
using namespace std;

void NosePiece::init( Microscope *_microscope,
                      const string &_readPositionCommand,
                      const string &_setPositionCommandRel,
                      const string &_setPositionCommand,
                      const string &_readMagnificationCommand )
  throw (mimas::mm_exception)
{
  microscope = _microscope;
  readPositionCommand = _readPositionCommand;
  setPositionCommandRel = _setPositionCommandRel;
  setPositionCommand = _setPositionCommand;
  readMagnificationCommand = _readMagnificationCommand;
}

int NosePiece::getPosition(void) const throw (mm_exception)
{
  assert( microscope != NULL );
  return microscope->readInt( readPositionCommand );
}

void NosePiece::setPositionRel( int shift ) throw (mm_exception)
{
  assert( microscope != NULL );
  microscope->sendInt( setPositionCommandRel, shift );
}

void NosePiece::setPosition( int position ) throw (mm_exception)
{
  assert( microscope != NULL );
  microscope->sendInt( setPositionCommand, position );
}

double NosePiece::getMagnification( int index ) const throw (mm_exception)
{
  assert( microscope != NULL );
  ostringstream command; command << readMagnificationCommand << ' ' << index;
  return microscope->readDouble( command.str() );
}
